import urllib
import json
import redis
import logging
import pymongo

from utils import parse_branch

from config import (
    LP_OUT_QUEUE, MONGO_HOST, MONGO_PORT,
    CHARM_IMPORT_FILTER, CHARM_IMPORT_LIMIT)

BRANCH_TIPS = "https://api.launchpad.net/devel/charm?ws.op=getBranchTips"
CHARM_DIR = ""

log = logging.getLogger("charm.launchpad")


def available_charms():
    charm_data = urllib.urlopen(BRANCH_TIPS).read()
    charm_data = json.loads(charm_data)

    count = 0

    for repo, commit, series in charm_data:
        _, branch_name = repo.rsplit("/", 1)

        data = parse_branch(repo, series, commit)
        if not data:
            log.warning("Unable to parse repo %s", repo)
            continue

        if CHARM_IMPORT_FILTER and not data['branch_spec'].startswith(
                CHARM_IMPORT_FILTER):
            continue

        if data["bname"] not in ("trunk", "trunk-1"):
            log.debug("Skipped branch %s", repo)
            continue

        log.info("Queueing %s", data['branch_spec'])
        yield data

        count += 1

        if CHARM_IMPORT_LIMIT and count >= CHARM_IMPORT_LIMIT:
            log.info("Import limit reached (%d)... stopping",
                     CHARM_IMPORT_LIMIT)
            break


def queue_charms(server, db):
    for charm in available_charms():
        added = server.sadd(LP_OUT_QUEUE, json.dumps(charm))
        if added and 0:
            log.info("Queued %s", charm)


def main():
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s: %(name)s@%(levelname)s: %(message)s")
    server = redis.Redis()
    db = pymongo.Connection(MONGO_HOST, MONGO_PORT)['juju']
    queue_charms(server, db)


if __name__ == '__main__':
    main()
