from datetime import datetime
import json
import logging
#import os
import redis
from urllib2 import urlopen

from config import STORE_IN_QUEUE, STORE_OUT_QUEUE, STORE_URL

log = logging.getLogger("charm.store")


def check_store(charm):
    if charm['owner'] == 'charmers':
        address = "cs:%s/%s" % (charm["series"], charm["name"])
    else:
        address = "cs:~%s/%s/%s" % (
            charm["owner"], charm["series"], charm["name"])

    data = _store_get(address)

    if 'errors' in data or 'warnings' in data:
        if charm['owner'] == 'charmers':
            log.info("rechecking %s with ~charmers", address)
            retry_address = "cs:~%s/%s/%s" % (
                charm["owner"], charm["series"], charm["name"])
            retry_data = _store_get(retry_address)
            if 1:
# Update the url to the user qualified name.
#            if not 'errors' in retry_data \
#               and not 'warnings' in retry_data:
                data = retry_data
                address = retry_address

    if 'errors' in data or 'warnings' in data:
        log.warning("store error on %s %s" % (address, data))

    data["store_checked"] = datetime.now().ctime()

    charm['store_data'] = data
    charm['store_url'] = address + "-%d" % data['revision']


def _store_get(address):
    url = STORE_URL + "/charm-info?charms=%s&stats=0" % address
    contents = urlopen(url).read()
    data = json.loads(contents)
    data = data[address]
    data['address'] = address
    return data


def run(server,
        in_queue=STORE_IN_QUEUE,
        out_queue=STORE_OUT_QUEUE):

    while 1:
        item = server.lpop(in_queue)
        if not item:
            return

        try:
            charm_data = json.loads(item)
            log.debug("Checking store for %s", charm_data["branch_spec"])
            check_store(charm_data)
        except Exception, e:
            log.exception("store error on %s: %s", item, str(e))
            continue
        else:
            server.lpush(out_queue, json.dumps(charm_data))


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.INFO,
        format="%(asctime)s: %(name)s@%(levelname)s: %(message)s")
    server = redis.Redis()
    run(server)
